/**
 * Copyright (C) 2010-2012 Agrosense info@agrosense.eu
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License"); you may
 * not use this file except in compliance with the License. You may obtain a
 * copy of the License at
 *
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package nl.cloudfarming.client.geometry.wizard;

import java.awt.Dimension;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.util.ArrayList;
import java.util.List;
import javax.swing.*;
import javax.swing.event.*;
import net.miginfocom.swing.MigLayout;
import nl.cloudfarming.client.geometry.model.GeometryContainer;
import nl.cloudfarming.client.geometry.model.Shape;
import nl.cloudfarming.client.util.swing.ASBodyPanel;
import org.openide.filesystems.FileUtil;
import org.openide.util.NbBundle;

/**
 * FilePicker panel to choose a file containing geometrical
 *
 * @author Merijn Zengers
 */
@NbBundle.Messages({
    "ImportShapeFilePickerPanel.file_label=Enter a file",
    "ImportShapeFilePickerPanel.noContainerLabel=There is no container to save shapes in. Please open or create a fields or taskcontainer.",
    "ImportShapeFilePickerPanel.browse_button.text=Browse...",
    "ImportShapeFilePickerPanel.browse_button.action_command=BROWSE",
    "ImportShapeFilePickerPanel.dialog.title=Choose a file",
    "ImportShapeFilePickerPanel open container text=The following geometry container types are avaialable for storing geometries",
    "ImportShapeFilePickerPanel select geometrycontainer label=Container",
    "ImportShapeFilePickerPanel.label.supports=Supports shape types: "
})
public class ImportShapeFilePickerPanel extends ASBodyPanel implements DocumentListener, ActionListener {

    private JLabel fileLabel;
    private JLabel noContainerLabel;
    private JTextField filePath;
    private javax.swing.JButton browseButton;
    private static final String BROWSE_BUTTON_TEXT = Bundle.ImportShapeFilePickerPanel_browse_button_text();
    private static final String BROWSE_BUTTON_ACTION_COMMAND = Bundle.ImportShapeFilePickerPanel_browse_button_action_command();
    private static final String DIALOG_TITLE = Bundle.ImportShapeFilePickerPanel_dialog_title();
    private static final String FILE_LABEL_TEXT = Bundle.ImportShapeFilePickerPanel_file_label();
    private static final String NO_CONTAINER_LABEL_TEXT = Bundle.ImportShapeFilePickerPanel_noContainerLabel();
    private static final int PATH_WIDTH = 400;
    private List<ChangeListener> changeListeners = new ArrayList<>();
    private DefaultComboBoxModel<GeometryContainer> defaultComboBoxModel = new DefaultComboBoxModel<>();
    private JComboBox containerBox;
    private JLabel supportsLabel = new JLabel(Bundle.ImportShapeFilePickerPanel_label_supports());
    private JLabel supportedTypes = new JLabel();
    private JLabel containerLabel = new JLabel(Bundle.ImportShapeFilePickerPanel_select_geometrycontainer_label());

    public ImportShapeFilePickerPanel() {
        super(new MigLayout("wrap 5"));
        init();

    }

    public final void init() {
        setIcon(ImportShapeWizardIterator.titleIcon);
        setTitle(DIALOG_TITLE);
        this.setName(DIALOG_TITLE);
        fileLabel = new JLabel(FILE_LABEL_TEXT);
        noContainerLabel = new JLabel(NO_CONTAINER_LABEL_TEXT);
       
        filePath = new JTextField();
        filePath.setPreferredSize(new Dimension(PATH_WIDTH, 0));
        filePath.getDocument().addDocumentListener(this);
        browseButton = new JButton(BROWSE_BUTTON_TEXT);
        browseButton.setActionCommand(BROWSE_BUTTON_ACTION_COMMAND);
        browseButton.addActionListener(new java.awt.event.ActionListener() {

            @Override
            public void actionPerformed(ActionEvent evt) {
                browseButtonActionPerformed(evt);
            }
        });

        // Set labels to bold
        Font containerBoldFont = new Font(containerLabel.getFont().getName(), Font.BOLD, containerLabel.getFont().getSize());
        containerLabel.setFont(containerBoldFont);
        noContainerLabel.setFont(containerBoldFont);
        Font supportsBoldFont = new Font(supportsLabel.getFont().getName(), Font.BOLD, supportsLabel.getFont().getSize());
        supportsLabel.setFont(supportsBoldFont);


        this.add(fileLabel);
        this.add(filePath, "span 3");
        this.add(browseButton);

        this.add(new JLabel(Bundle.Import_geometrical_wizard_panel_assign_to_fieldproject_message()), "span, wrap, gaptop 15");

        this.add(containerLabel, "gaptop 10");
        containerBox = new JComboBox(defaultComboBoxModel);
        containerBox.addActionListener(this);
        this.add(containerBox);
        this.add(supportsLabel, "gapleft 10");
        this.add(supportedTypes, "span");

        
        this.add(noContainerLabel, "span, wrap, gaptop 10");
        noContainerLabel.setVisible(false);
    }

    /**
     * Set available field containers for every shape type
     *
     * @param shapeTypeContainers a list containing GeometryContainer
     */
    public void setAvailableFieldContainers(List<GeometryContainer> containers) {
        defaultComboBoxModel.removeAllElements();
        for (GeometryContainer fieldContainer : containers) {
            defaultComboBoxModel.addElement(fieldContainer);
            noContainerLabel.setVisible(false);
        }
    }

    private void browseButtonActionPerformed(ActionEvent evt) {
        String command = evt.getActionCommand();
        if (BROWSE_BUTTON_ACTION_COMMAND.equals(command)) {
            JFileChooser chooser = new JFileChooser();
            chooser.setCurrentDirectory(null);
            chooser.setDialogTitle(DIALOG_TITLE);
            chooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
            String path = this.filePath.getText();
            if (path.length() > 0) {
                File f = new File(path);
                if (f.exists()) {
                    chooser.setSelectedFile(f);
                }
            }
            if (JFileChooser.APPROVE_OPTION == chooser.showOpenDialog(this)) {
                File projectDir = chooser.getSelectedFile();
                filePath.setText(FileUtil.normalizeFile(projectDir).getAbsolutePath());
            }
        }
    }

    /**
     * Get the path of the chosen file
     *
     * @return a String containing the path to the chosen file
     */
    protected String getChosenFilePath() {
        return filePath.getText();
    }

    /**
     * Set the path of the file
     *
     * @param path the path to set
     */
    protected void setChosenFilePath(String path) {
        filePath.setText(path);
    }

    /**
     *
     * @return Selected fieldContainer
     */
    public GeometryContainer getSelectedGeoMetryContainer() {
        // Casting is save, model only has fieldContainers
        return (GeometryContainer) defaultComboBoxModel.getSelectedItem();
    }

    public void addChangeListener(ChangeListener l) {
        changeListeners.add(l);
    }

    public void removeChangeListener(ChangeListener l) {
        changeListeners.remove(l);
    }

    private void fireChange() {
        for (ChangeListener listener : changeListeners) {
            listener.stateChanged(new ChangeEvent(this));
        }
    }

    @Override
    public void insertUpdate(DocumentEvent de) {
        fireChange();
    }

    @Override
    public void removeUpdate(DocumentEvent de) {
        fireChange();
    }

    @Override
    public void changedUpdate(DocumentEvent de) {
        fireChange();
    }

    @Override
    public void actionPerformed(ActionEvent e) {
        JComboBox cb = (JComboBox) e.getSource();
        if (cb.getSelectedItem() != null) {
            GeometryContainer geometryContainer = (GeometryContainer) cb.getSelectedItem();
            supportedTypes.setText(getSupportsString(geometryContainer));
        }
    }

    public String getSupportsString(GeometryContainer geometryContainer) {
        // TODO: Adjust to show more then one support
        Shape.ShapeType shapeType = geometryContainer.getShapeType();
        return shapeType.getName();
    }
    
    public void hideContainer() {
        containerBox.setVisible(false);
        containerLabel.setVisible(false);
        fileLabel.setVisible(false);
        filePath.setVisible(false);
        browseButton.setVisible(false);
        supportedTypes.setVisible(false);
        supportsLabel.setVisible(false);
        
        noContainerLabel.setVisible(true);
    }
}
