/**
 * Copyright (C) 2010-2012 Agrosense info@agrosense.eu
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License"); you may
 * not use this file except in compliance with the License. You may obtain a
 * copy of the License at
 *
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package nl.cloudfarming.client.geometry.wizard;

import java.awt.Component;
import java.awt.Dialog;
import java.awt.event.ActionEvent;
import java.io.File;
import java.text.MessageFormat;
import java.util.ArrayList;
import java.util.List;
import java.util.NoSuchElementException;
import javax.swing.AbstractAction;
import javax.swing.ImageIcon;
import javax.swing.JComponent;
import javax.swing.event.ChangeListener;
import nl.cloudfarming.client.geometry.GeoImportSupport;
import nl.cloudfarming.client.geometry.PreviewPanel;
import nl.cloudfarming.client.geometry.model.GeometryContainer;
import nl.cloudfarming.client.geometry.model.Shape;
import org.openide.DialogDisplayer;
import org.openide.WizardDescriptor;
import org.openide.awt.ActionID;
import org.openide.awt.ActionReference;
import org.openide.awt.ActionReferences;
import org.openide.awt.ActionRegistration;
import org.openide.filesystems.FileObject;
import org.openide.filesystems.FileUtil;
import org.openide.loaders.DataObject;
import org.openide.loaders.DataObjectNotFoundException;
import org.openide.util.Exceptions;
import org.openide.util.ImageUtilities;
import org.openide.util.NbBundle;

/**
 * Dynamic wizard. Wizard will find a suitable preview panel (Step2) for the
 * selected using the geometrical file selected in the first step.
 *
 * @author Merijn Zengers & Gerben Feenstra
 */
@NbBundle.Messages({
    "Import.geometrical.wizard.step.filepicker.text=Pick a file",
    "Import.geometrical.wizard.step.preview=Preview",
    "Import.geometrical.wizard.step.summary=Overview",
    "Import.geometrical.wizard.name=Import geometries"
})
@ActionID(category = "Project", id = "nl.cloudfarming.client.geometry.ImportAction")
@ActionRegistration(iconBase = "nl/cloudfarming/client/geometry/map_add.png", displayName = "#Import.geometrical.wizard.name")
@ActionReferences({
    @ActionReference(path = "Menu/File", position = 0)
})
public class ImportShapeWizardIterator extends AbstractAction implements WizardDescriptor.Iterator {

    private int index;
    private List<WizardDescriptor.Panel> panels;
    private WizardDescriptor wizardDescriptor;
    private static final String TITLE = NbBundle.getMessage(ImportShapeWizardIterator.class, "Import.geometrical.wizard.name");
    public static final ImageIcon titleIcon = new ImageIcon(ImageUtilities.loadImage("nl/cloudfarming/client/geometry/map_add32.png"));

    public ImportShapeWizardIterator() {
    }

    protected List<WizardDescriptor.Panel> getPanels() {
        return panels;
    }

    /**
     * Gets the current panel from this wizardIterator
     * @return Current Panel
     */
    @Override
    public WizardDescriptor.Panel current() {
        return getPanels().get(index);
    }

    /**
     * Gets the steps this wizard contains
     * @return a string array containing the title of every step
     */
    private String[] getSteps(){
        return new String[]{
            Bundle.Import_geometrical_wizard_step_filepicker_text(),
            Bundle.Import_geometrical_wizard_step_preview(),
            Bundle.Import_geometrical_wizard_step_summary()};
    }
    
    /**
     * Set properties for each Panel in the list
     */
    private void setPanelProperties() {
        for (int i = 0; i < getPanels().size(); i++) {
            WizardDescriptor.Panel panel = getPanels().get(i);
            Component c = panel.getComponent();
            if (c instanceof JComponent) {
                JComponent jc = (JComponent) c;
                // Sets step number of a component
                jc.putClientProperty("WizardPanel_contentSelectedIndex", Integer.valueOf(i));
                // Sets steps names for a panel
                jc.putClientProperty("WizardPanel_contentData", getSteps());
                // Turn on subtitle creation on each step
                jc.putClientProperty("WizardPanel_autoWizardStyle", Boolean.TRUE);
                // Show steps on the left side with the image on the background
                jc.putClientProperty("WizardPanel_contentDisplayed", Boolean.TRUE);
                // Turn on numbering of all steps
                jc.putClientProperty("WizardPanel_contentNumbered", Boolean.TRUE);
            }
        }
    }

    @Override
    public String name() {
        return MessageFormat.format("{0} of {1}",
                new Object[]{new Integer(index + 1), new Integer(getPanels().size())});
    }

    @Override
    public boolean hasNext() {
        if (index == 0) {
            return true;
        }
        return index < getPanels().size() - 1;
    }

    @Override
    public boolean hasPrevious() {
        return index > 0;
    }

    /**
     * Determine dataObject selected in first panel
     */
    protected DataObject getSelectedDataObject() {
        String filePath = (String) wizardDescriptor.getProperty(ImportShapeFilePicker.FILE_PATH);
        FileObject fileObj = FileUtil.toFileObject(new File(filePath));
        DataObject dataObject = null;
        try {
            dataObject = DataObject.find(fileObj);
        } catch (DataObjectNotFoundException ex) {
            Exceptions.printStackTrace(ex);
        }
        return dataObject;
    }

    @Override
    public void nextPanel() {
        // Second panel depends on outcome of first panel
        if (index == 0) {
            // Retrieve selected dataObject
            DataObject dataObject = getSelectedDataObject();
            assert (dataObject != null);

            // Find suitable panel to display the selected geometry
            WizardDescriptor.Panel secondPanel = GeoImportSupport.Util.findPreviewPanelFor(dataObject);

            if (secondPanel != null) {
                getPanels().set(1, secondPanel); // Replace dummy Panel
                // Refresh panel properties such as current stepnr and available steps
                JComponent jc = (JComponent) secondPanel.getComponent();
                // Sets step number of a component
                jc.putClientProperty("WizardPanel_contentSelectedIndex", Integer.valueOf(1));
                // Sets steps names for a panel
                jc.putClientProperty("WizardPanel_contentData", getSteps());
                // Turn on subtitle creation on each step
                jc.putClientProperty("WizardPanel_autoWizardStyle", Boolean.TRUE);
                // Show steps on the left side with the image on the background
                jc.putClientProperty("WizardPanel_contentDisplayed", Boolean.TRUE);
                // Turn on numbering of all steps
                jc.putClientProperty("WizardPanel_contentNumbered", Boolean.TRUE);
            }
        }

        if (!hasNext()) {
            throw new NoSuchElementException();
        }
        index++;
    }

    @Override
    public void previousPanel() {
        if (!hasPrevious()) {
            throw new NoSuchElementException();
        }
        index--;
    }

    @Override
    public void addChangeListener(ChangeListener l) {
    }

    @Override
    public void removeChangeListener(ChangeListener l) {
    }

    @Override
    public void actionPerformed(ActionEvent ae) {
        panels = new ArrayList<>();
        panels.add(new ImportShapeFilePicker());
        panels.add(new GeoImportStubControllerPanel()); // Dummy panel, find suitable panel after first step
        panels.add(new ImportShapeSummary());
        setPanelProperties();

        index = 0;
        wizardDescriptor = new WizardDescriptor(this);

        // {0} will be replaced by WizardDescriptor.Panel.getComponent().getName()
        wizardDescriptor.setTitleFormat(new MessageFormat("{0}"));
        wizardDescriptor.setTitle(TITLE);

        Dialog dialog = DialogDisplayer.getDefault().createDialog(wizardDescriptor);
        dialog.setVisible(true);
        dialog.toFront();

        boolean cancelled = wizardDescriptor.getValue() != WizardDescriptor.FINISH_OPTION;
        if (!cancelled) {
            // DO SAVE STUFF
            DataObject selectedDataObject = getSelectedDataObject();
            List<Shape> shapes = (List<Shape>) wizardDescriptor.getProperty(PreviewPanel.IMPORTED_SHAPES);
            GeometryContainer geometryContainer = (GeometryContainer) wizardDescriptor.getProperty(ImportShapeFilePicker.PROP_SELECTED_GEOMETRYCONTAINER);

            List<Shape> supportedShapes = new ArrayList<>();
            for (Shape shape : shapes) {
                assert shape.getType() != null : "The Shape.ShapeType of shape must be set";
                if (geometryContainer.supports(shape.getType())) {
                    supportedShapes.add(shape);
                }
            }
            
            geometryContainer.importShapes(supportedShapes);
            geometryContainer.importSources(selectedDataObject);
        }
    }
}
